using System;
using Microsoft.Practices.Unity;
using Benday.UserGroupManager.Common;

namespace Benday.UserGroupManager.Factories
{
    public class ObjectFactory : IObjectFactory
    {
        private ObjectFactoryConfiguration m_additionalConfiguration;

        public ObjectFactory()
        {
            m_additionalConfiguration = null;
        }

        public ObjectFactory(ObjectFactoryConfiguration configuration)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException("configuration", "Argument cannot be null.");
            }

            m_additionalConfiguration = configuration;
        }

        private UnityContainer m_container;
        private UnityContainer Container
        {
            get
            {
                if (m_container == null)
                {
                    InitializeContainer("ObjectFactory");
                }

                return m_container;
            }
            set { m_container = value; }
        }

        private void InitializeContainer(string containerConfigName)
        {
            UnityContainer container = new UnityContainer();

            container.RegisterInstance<IObjectFactory>(this);

            if (containerConfigName == "UNIT_TESTS")
            {
                //container.RegisterInstance<ICmsLinkRepository>(new CmsLinkRepositoryMock());                
            }
            else
            {

            }

            if (m_additionalConfiguration != null)
            {
                RegisterAdditionalConfiguration(container, m_additionalConfiguration);
            }

            m_container = container;
        }

        private void RegisterAdditionalConfiguration(UnityContainer container, ObjectFactoryConfiguration configuration)
        {
            if (container == null)
                throw new ArgumentNullException("container", "container is null.");
            if (configuration == null)
                throw new ArgumentNullException("configuration", "Configuration is null.");

            foreach (Type key in configuration.TypeMappings.Keys)
            {
                container.RegisterType(key, configuration.TypeMappings[key]);
            }

            foreach (Type key in configuration.Instances.Keys)
            {
                container.RegisterInstance(key, configuration.Instances[key]);
            }
        }

        public void RegisterInstance<T>(T instance)
        {
            Container.RegisterInstance<T>(instance);
        }

        public T Create<T>()
        {
            try
            {
                return Container.Resolve<T>();
            }
            catch (ResolutionFailedException ex)
            {
                throw new ObjectFactoryException(
                    String.Format("Problem creating instance of type '{0}'.", typeof(T).FullName),
                    ex);
            }
        }

        public void Reconfigure(string configurationName)
        {
            InitializeContainer(configurationName);
        }
    }
}
