using System;
using System.ComponentModel;
using GalaSoft.MvvmLight;
using System.Collections.Generic;
using Benday.UserGroupManager.Common;

namespace Benday.UserGroupManager.Presentation
{
    public class GetByIdCompletedEventArgs
    {
        public GetByIdCompletedEventArgs()
        {
            
        }
    }

    public class PersonRepository : IPersonRepository
    {
        public void GetById(ReturnResult<IPerson> callback, int id)
        {
            PersonServiceClient service = null;

            try
            {
                // create an instance of the WCF PersonServiceClient proxy
                service = new PersonServiceClient();

                // subscribe to the completed event for GetById()
                service.GetByIdCompleted +=
                    new EventHandler<GetByIdCompletedEventArgs>(
                        service_GetByIdCompleted);

                // call the GetById service method
                // pass "callback" as the userState value to the call
                service.GetByIdAsync(id, callback);
            }
            catch
            {
                if (service != null) service.Abort();
                throw;
            }
        }

        void service_SearchCompleted(object sender, GetByIdCompletedEventArgs e)
        {
            var callback = e.UserState as ReturnResult<IList<IPerson>>;

            if (callback == null)
            {
                // this is bad
                throw new InvalidOperationException(
                    "e.UserState was not an instance of ReturnResult<IPerson>.");
            }

            if (e.Error != null)
            {
                // something went wrong
                // "throw" the exception up the call stack
                callback.Notify(e.Error);
                return;
            }

            try
            {
                IPerson returnValue;

                // do whatever needs to be done to create and populate an
                // instance of IPerson

                callback.Notify(returnValue);
            }
            catch (Exception ex)
            {
                callback.Notify(ex);
            }
        }
    }
}
