using System;
using System.ComponentModel;
using GalaSoft.MvvmLight;
using System.Collections.Generic;
using Benday.UserGroupManager.Common;

namespace Benday.UserGroupManager.Presentation
{
    public class PersonViewModel
    {
        public void LoadById(int id)
        {
            IPersonRepository repository = GetRepositoryInstance();

            // initiate the call to the repository
            repository.GetById(new ReturnResult<IPerson>(LoadByIdCompleted), id);
        }

        private void LoadByIdCompleted(ReturnResult<IPerson> result)
        {
            // handle the callback from the repository

            if (result == null)
            {
                throw new InvalidOperationException("Result was null.");
            }
            else if (result.Error != null)
            {
                HandleTheException(result.Error);
            }
            else
            {
                IPerson person = result.Result;

                if (person == null)
                {
                    // person not found...invalid id?
                    Id = String.Empty;
                    FirstName = String.Empty;
                    LastName = String.Empty;
                    EmailAddress = String.Empty;
                }
                else
                {
                    Id = person.Id;
                    FirstName = person.FirstName;
                    LastName = person.LastName;
                    EmailAddress = person.EmailAddress;
                }
            }
        }

        private void HandleTheException(Exception error)
        {
            throw new NotImplementedException();
        }

        private IPersonRepository GetRepositoryInstance()
        {
            throw new NotImplementedException();
        }

        public string Id { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string EmailAddress { get; set; }
    }
}
