using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GalaSoft.MvvmLight;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.UserGroupManager.Common;
using Benday.UserGroupManager.Presentation;
using System.ComponentModel;
using System.Collections.Generic;

namespace Benday.UserGroupManager.UnitTests.Presentation
{
    public class NotifyPropertyChangedTester
    {
        public NotifyPropertyChangedTester(INotifyPropertyChanged viewModel)
        {
            if (viewModel == null)
            {
                throw new ArgumentNullException("viewModel", "Argument cannot be null.");
            }

            this.Changes = new List<string>();

            viewModel.PropertyChanged += new PropertyChangedEventHandler(viewModel_PropertyChanged);
        }

        void viewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            Changes.Add(e.PropertyName);
        }

        public List<string> Changes { get; private set; }

        public void AssertChange(int changeIndex, string expectedPropertyName)
        {
            Assert.IsNotNull(Changes, "Changes collection was null.");

            Assert.IsTrue(changeIndex < Changes.Count,
                "Changes collection contains '{0}' items and does not have an element at index '{1}'.",
                Changes.Count,
                changeIndex);

            Assert.AreEqual<string>(expectedPropertyName,
                Changes[changeIndex],
                "Change at index '{0}' is '{1}' and is not equal to '{2}'.",
                changeIndex,
                Changes[changeIndex],
                expectedPropertyName);
        }
    }
}
