﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.UserGroupManager.Presentation;
using System.ComponentModel;

namespace Benday.UserGroupManager.UnitTests.Presentation
{
    [TestClass]
    public class ViewModelFieldFixture : SilverlightUnitTestBase
    {
        [TestMethod]
        public void ViewModelField_Constructor_Empty()
        {
            var instance = new ViewModelField<string>();

            Assert.IsNull(instance.Value, "Value should be null.");
        }

        [TestMethod]
        public void ViewModelField_Constructor_WithValue()
        {
            string expectedValue = "expected value";

            var instance = new ViewModelField<string>(expectedValue);

            Assert.AreEqual<string>(expectedValue, instance.Value, "Value was wrong.");
        }

        [TestMethod]
        public void ViewModelField_ToString_ReturnsEmptyStringWhenValueIsNull()
        {
            var instance = new ViewModelField<string>();

            Assert.IsNull(instance.Value, "Value should be null.");
            Assert.AreEqual<string>(String.Empty, instance.ToString(), "ToString() should return empty.");
        }

        [TestMethod]
        public void ViewModelField_ToString()
        {
            string expectedValue = "expected value";

            var instance = new ViewModelField<string>(expectedValue);

            Assert.AreEqual<string>(expectedValue, instance.Value, "Value was wrong.");
            Assert.AreEqual<string>(expectedValue, instance.ToString(), "ToString() was wrong.");
        }

        [TestMethod]
        public void ViewModelField_NotifyPropertyChanged_NotifiesOnNewValue()
        {
            string originalValue = "original value";

            var instance = new ViewModelField<string>(originalValue);

            Assert.IsInstanceOfType(instance, typeof(INotifyPropertyChanged));

            bool gotEvent = false;

            ((INotifyPropertyChanged)instance).PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                gotEvent = true;
                Assert.AreEqual<string>("Value", e.PropertyName, "PropertyName was wrong.");
            };            
            
            string newValue = "new value";
            instance.Value = newValue;

            Assert.AreEqual<string>(newValue, instance.Value, "Value didn't change.");

            Assert.IsTrue(gotEvent, "Didn't get the PropertyChanged event.");
        }

        [TestMethod]
        public void ViewModelField_NotifyPropertyChanged_DoesNotNotifyOnSameValue()
        {
            string originalValue = "original value";

            var instance = new ViewModelField<string>(originalValue);

            Assert.IsInstanceOfType(instance, typeof(INotifyPropertyChanged));

            bool gotEvent = false;

            ((INotifyPropertyChanged)instance).PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                gotEvent = true;
                Assert.Fail("Should not get any PropertyChanged events.");
            };

            instance.Value = originalValue;

            Assert.IsFalse(gotEvent, "Should not have gotten the PropertyChanged event.");
        }
    }
}
