﻿using System;
using Rhino.Mocks;
using Microsoft.Silverlight.Testing;
using Benday.UserGroupManager.Common;
using Benday.UserGroupManager.Factories;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Benday.UserGroupManager.UnitTests
{
    public abstract class SilverlightUnitTestBase : SilverlightTest
    {
        private ObjectFactoryConfiguration m_additionalObjectFactoryConfiguration;
        public ObjectFactoryConfiguration AdditionalObjectFactoryConfiguration
        {
            get
            {
                if (m_additionalObjectFactoryConfiguration == null)
                {
                    m_additionalObjectFactoryConfiguration = new ObjectFactoryConfiguration();

                    PopulateAdditionalObjectFactoryConfiguration(m_additionalObjectFactoryConfiguration);
                }

                return m_additionalObjectFactoryConfiguration;
            }
        }

        protected virtual void PopulateAdditionalObjectFactoryConfiguration(ObjectFactoryConfiguration configuration)
        {

        }

        private IObjectFactory m_objectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory(AdditionalObjectFactoryConfiguration);
                }

                return m_objectFactoryInstance;
            }
        }

        public void ResetObjectFactoryInstance()
        {
            m_objectFactoryInstance = null;
        }

        protected MockRepository m_mockRepositoryInstance;
        public MockRepository MockRepositoryInstance
        {
            get
            {
                if (m_mockRepositoryInstance == null)
                {
                    m_mockRepositoryInstance = new MockRepository();
                }

                return m_mockRepositoryInstance;
            }
        }

        protected virtual void OnTestInitialize()
        {

        }

        [TestInitialize]
        public void TestInitialize()
        {
            OnTestInitialize();
        }

        protected virtual void OnClassInitialize()
        {
            m_objectFactoryInstance = null;
            m_additionalObjectFactoryConfiguration = null;
        }

        [ClassInitialize]
        public void ClassInitialize()
        {
            OnClassInitialize();
        }
    }
}